#include-once

Global Const $MAX_FIELDS = 256
Global Enum $FIELD_TYPE, $FIELD_NAME, $FIELD_UNIT

Global Const $ft_nomorefields = 0 
Global Const $ft_numeric_32 = 1 
Global Const $ft_numeric_64 = 2 
Global Const $ft_numeric_floating = 3 
Global Const $ft_date = 4 
Global Const $ft_time = 5 
Global Const $ft_boolean = 6 
Global Const $ft_multiplechoice = 7 
Global Const $ft_string = 8 
Global Const $ft_fulltext = 9 
Global Const $ft_datetime = 10 
Global Const $ft_stringw = 11 

Global Const $ft_nosuchfield = -1
Global Const $ft_fileerror = -2
Global Const $ft_fieldempty = -3
Global Const $ft_ondemand = -4

Func _PHWDXGetFields($sWdxPlugin)
    Local $aFields[$MAX_FIELDS][3], $nFiledCount = 0
    For $i = 0 To $MAX_FIELDS - 1 
        $aRet = DllCall($sWdxPlugin, "int", "ContentGetSupportedField", "int", $i, "str", "", "str", "", "int", 65535) 
        If @error Then ExitLoop 
        If $aRet[0] = $ft_nomorefields Then ExitLoop
        $aFields[$i][$FIELD_TYPE] = $aRet[0]
        $aFields[$i][$FIELD_NAME] = $aRet[2]
        $aFields[$i][$FIELD_UNIT] = $aRet[3]
        $nFiledCount += 1
    Next
    If $nFiledCount = 0 Then Return SetError(1,0,"")
    ReDim $aFields[$nFiledCount][3]
    Return $aFields
EndFunc

Func _PHWDXGetValues($sPlugin, $sFileName)
    Local $hPlugin = DllOpen($sPlugin)
    If @Error Then Return SetError(1,0,"")

    Local $aFieldValues[$MAX_FIELDS], $nFieldCount = 0
    For $i = 0 To $MAX_FIELDS - 1
        $vFieldValue = __PHWDXGetFieldValue($hPlugin, $sFileName, $i, 0)
        If @error Then ExitLoop
        $aFieldValues[$i] = $vFieldValue
        $nFieldCount += 1
    Next

    DllClose($hPlugin)
    
    If $nFieldCount > 0 Then 
        ReDim $aFieldValues[$nFieldCount]
        Return $aFieldValues
    Else
        Return SetError(1, 0, "")
    EndIf
EndFunc

Func _PHWDXGetValue($sPlugin, $sFileName, $nFieldIdx, $nUnitIdx = 0)
    Local $error
    Local $hPlugin = DllOpen($sPlugin)
    If @Error Then Return SetError(1,0,"")
    Local $vFieldValue = __PHWDXGetFieldValue($hPlugin, $sFileName, $nFieldIdx, $nUnitIdx)
    If @error Then
        DllClose($hPlugin)
        Return SetError(@error, 0, "")
    Else
        DllClose($hPlugin)
        Return $vFieldValue
    EndIf
EndFunc


Func __PHWDXGetFieldValue(ByRef $hPlugin, $sFileName, $nFieldIdx, $nUnitIdx)
    Local $nMaxBuffSize = 65535 
    Local $pBuffer = DllStructCreate("byte[" & $nMaxBuffSize & "]")
    $aRet = DllCall($hPlugin, "int", "ContentGetValue", "str", $sFileName, "int", $nFieldIdx, "int", $nUnitIdx, "ptr", DllStructGetPtr($pBuffer), "int", $nMaxBuffSize, "int", 0) 
    If @error Then Return SetError(1, 0, "")
    Switch $aRet[0]
        Case $ft_numeric_32 
            $pValue = DllStructCreate("int", DllStructGetPtr($pBuffer)) 
        Case $ft_numeric_64 
            $pValue = DllStructCreate("int64", DllStructGetPtr($pBuffer)) 
        Case $ft_numeric_floating 
            $pValue = DllStructCreate("double", DllStructGetPtr($pBuffer)) 
        Case $ft_string, $ft_fulltext 
            $pValue = DllStructCreate("char[" & $nMaxBuffSize & "]", DllStructGetPtr($pBuffer)) 
        Case $ft_stringw 
            $pValue = DllStructCreate("wchar[" & $nMaxBuffSize / 2 & "]", DllStructGetPtr($pBuffer)) 
        Case $ft_datetime 
            $pFileTime = DllStructCreate("dword;dword", DllStructGetPtr($pBuffer)) 
            $pLocalTime = DllStructCreate("dword;dword") 
            DllCall("Kernel32.dll", "int", "FileTimeToLocalFileTime", "ptr", DllStructGetPtr($pFileTime), "ptr", DllStructGetPtr($pLocalTime)) 
            $pValue = DllStructCreate("short;short;short;short;short;short;short;short") 
            DllCall("Kernel32.dll", "int", "FileTimeToSystemTime", "ptr", DllStructGetPtr($pLocalTime), "ptr", DllStructGetPtr($pValue)) 
            Return StringFormat("%04d-%02d-%02d %02d:%02d:%02d", _ 
                                DllStructGetData($pValue, 1), _ 
                                DllStructGetData($pValue, 2), _ 
                                DllStructGetData($pValue, 4), _ 
                                DllStructGetData($pValue, 5), _ 
                                DllStructGetData($pValue, 6), _ 
                                DllStructGetData($pValue, 7))
        Case $ft_nosuchfield, $ft_fileerror, $ft_fieldempty
            Return SetError(2, 0, "")
        Case Else
            Return SetError(3, 0, "")
    EndSwitch 
    Return DllStructGetData($pValue, 1)
EndFunc
